import type { Constructor } from 'web-utility';
import type { CustomElement } from 'web-utility';
import { WebCellProps, WebCellElement } from './utility';
import { ComponentMeta, DOMEventDelegater } from './decorator';
import { VNodeChildElement } from './renderer';
export declare type WebCellFunction<P extends WebCellProps = WebCellProps> = (props?: P) => WebCellElement;
export interface WebCellComponent<P extends WebCellProps = WebCellProps, S = {}> extends CustomElement {
    root: DocumentFragment | Element;
    update(): void;
    props: P;
    setProps(data: Partial<P>): Promise<any>;
    state: S;
    setState(data: Partial<S>): Promise<void>;
    defaultSlot: VNodeChildElement[];
    render(props: P, state: S): WebCellElement;
    /**
     * Called before `state` is updated
     */
    shouldUpdate?(oldState: S, newState: S): boolean;
    /**
     * Called after rendering
     */
    updatedCallback?(): void;
    emit(event: string, detail?: any, options?: EventInit): boolean;
    toString(): string;
}
export interface WebCellClass<P extends WebCellProps = WebCellProps, S = {}> extends Partial<ComponentMeta>, Constructor<WebCellComponent<P, S>> {
    attributes?: string[];
    eventDelegaters?: DOMEventDelegater[];
}
export declare function mixin<P extends WebCellProps = WebCellProps, S = {}>(superClass?: Constructor<HTMLElement>): WebCellClass<P, S>;
